# Generated by roxygen2: do not edit by hand

S3method(as_graph,graph)
S3method(as_graph,list)
S3method(blockr.core::blockr_deser,subboard)
S3method(blockr.core::blockr_ser,subboard)
S3method(board_actions,dag_extension)
S3method(context_menu_items,dag_extension)
S3method(context_menu_items,dock_board)
S3method(context_menu_items,dock_extension)
S3method(context_menu_items,dock_extensions)
S3method(context_menu_items,list)
S3method(extension_block_callback,dag_extension)
S3method(toolbar_items,dag_extension)
S3method(toolbar_items,dock_board)
S3method(toolbar_items,dock_extension)
S3method(toolbar_items,dock_extensions)
S3method(toolbar_items,list)
export(as_graph)
export(context_menu_items)
export(graph_combos)
export(graph_edges)
export(graph_nodes)
export(is_context_menu_entry)
export(is_graph)
export(is_toolbar_item)
export(new_context_menu_entry)
export(new_dag_extension)
export(new_graph)
export(new_toolbar_item)
export(toolbar_items)
import(blockr.core)
import(g6R)
import(shiny)
importFrom(blockr.dock,board_actions)
importFrom(blockr.dock,extension_block_callback)
